@@ -23,7 +23,7 @@ data = (
                 notes=[],
             )
         },
-        global_timing=None,
+        common_timing=None,
     ),
     False,
 )
