@@ -3,7 +3,7 @@ from hypothesis import strategies as st
 
 from jubeatools import song
 from jubeatools.formats.konami.commons import EveLong
-from jubeatools.formats.konami.timemap import TimeMap
+from jubeatools.formats.timemap import TimeMap
 from jubeatools.testutils import strategies as jbst
 
 
